﻿/*************************************************************************************************
 * Velleman K8000 Class for FEZ Panda II.
 * Class Version    : 2.0 
 * Released         : 27 April 2012.
 * 
 *  Copyright (c) 2011-2012 - Vanderhaegen Bart
 * 
 * 
 * I have written this class to be able to control the Velleman K8000 Interface Card
 * from the GHI Electronics FEZ Panda II board. The code uses the I2C functionality
 * of the Panda II board.
 * 
 * The code doesn't require any specific FEZ libraries because it uses the standard I2C
 * functions that are provided in the Microsoft.SPOT libraries. So ot will probably
 * work with other GHI products as well, but your product must have a native support
 * for the I2C bus.
 * 
 * This source code was originally written by Velleman, in a BASIC language. I have
 * converted this into a C# code. Most of the functions act exactly the same as in
 * the original Velleman code.
 * 
 * 
 * This code comes at it is. I cannot be held responsible for possible errors in
 * this code. I only tested the functions I need the most (I/O output operations).
 * In case of problems and/or questions you can contact me at the following E-mail
 * address:
 * 
 *  bart.vanderhaegen@bvsystems.be
 *  http://www.bvsystems.be
 * 
 * 
 * 
 * Basic Usage:
 * ------------
 * 
 * You only need to add this class to your FEZ project. Then you can create an 
 * object of this class. Using this object you can control the K8000. Example:
 * 
 * Velleman_K8000 k8000 = new Velleman_K8000(1, true);
 * k8000.setAllIO();
 * 
 * The constructor of the class is taking 2 parameters:
 * 1) Number of K8000 cards you have (1-4). Only enter the actual number of cards!
 * 2) A boolean to indicate if you wish to generate an Exception if an I2C error
 *   occured (this can be usefull to solve bugs).
 * 
 * 
 * Attention!
 * ----------
 * 
 * ATTENTION!!! The FEZ is connected with the K8000 by means of the SCL/SDA/GND
 * lines. The K8000 runs I2C with voltages of 5 VDC. You need to make sure your
 * FEZ product can handle 5V at the input of the SCL/SDA ports! (Most processors
 * run on 3.3 VDC).
 * 
 * The FEZ Panda II can handle 5 VDC at the inputs of the SCL/SDA lines so the
 * K8000 can be connected to the FEZ directly.
 * 
 * On the Internet you can find schematics on how you can connect I2C systems that
 * are operating on different voltages (3.3V <-> 5V, ...).
 * 
 *************************************************************************************************/

using System;                       // only really needed if you want to use the Debug keyword.
using Microsoft.SPOT.Hardware;
using System.IO;
using Microsoft.SPOT;


/// <summary>
/// This class allows you to control the Velleman_K8000 from the FEZ Panda II.
/// </summary>
public class Velleman_K8000
{
    // Constants with information about the specifications of the cards
    private int MAX_IO_CARD;
    private int MAX_IO_CHIP;
    private int MAX_IO_CHANNEL;
    private int MAX_DAC_CHANNEL;
    private int MAX_AD_CHANNEL;
    private int MAX_DA_CHANNEL;

    /// <summary>
    /// IO is an array that contains information about the state of all I/O channels.
    /// </summary>
    public int[] IO;

    /// <summary>
    /// DAC is an array that contains information about the state of all DAC channels.
    /// </summary>
    public int[] DAC;

    /// <summary>
    /// AD is an array that contains information about the state of all AD channels.
    /// </summary>
    public int[] AD;

    /// <summary>
    /// DA is an array that contains information about the state of all DA channels.
    /// </summary>
    public int[] DA;



    private int[] IOconfig;
    private int[] IOdata;
    private int[] ADDAchipCode;
    private int[] DACchipCode;
    private int[] IOchipCode;

    // Settings for the I2C communications
    private const int I2C_BUS_FREQUENCY = 50;           // the frequency the I2C bus should be using
    private bool ERROR_ON_FAILURE;                      // generate an exception when an I2C error occurs?
    private bool ERROR_DURING_LAST_I2C_ACTION;          // did the last I2C action return any errors?




    #region "Initialisation"
        /// <summary>
        /// init() will initialize the k8000 card. This function MUST be executed before
        /// you can use any other function.
        /// </summary>
        public void init()
        {   // here the I2C chipcodes will be calculated.
            for (int cardNo = 0; cardNo < MAX_IO_CARD; cardNo++)
            {
                DACchipCode[cardNo] = (64 + 2 * cardNo) >> 1;
                ADDAchipCode[cardNo] = (144 + 2 * cardNo) >> 1;
            }

            for (int chipNo = 0; chipNo < MAX_IO_CHIP; chipNo++)
            {
                IOchipCode[chipNo] = (byte)(112 + 2 * chipNo) >> 1;
            }

            configAllIOasOutput();
            clearAllDAC();
            clearAllDA();
            readAll();
        }



        /// <summary>
        /// getIOstr() returns a string that contains the binary value of all I/O channels.
        /// The function only works for card 0. The function doesn't return the values of 
        /// the K8000 card first.
        /// </summary>
        /// <returns></returns>
        public string getIOstr()
        {
            readAllIO();
            string response = "";

            for (int i = 0; i < 16; i++)
                response += IO[i+1];
        
            return response;
        }





        /// <summary>
        ///  Constructor to create the K8000 object.
        /// </summary>
        /// <param name="numberOfCards">specify the number of k8000 cards you have connected (1-4)</param>
        /// <param name="errorOnI2Cfailure">Specify if you want to throw an exception when an I2C error occured 
        /// (this will happen when a chip was not found)</param>
        public Velleman_K8000(int numberOfCards,bool errorOnI2Cfailure)
        {   // check if the number of cards is within the allowed range.
            if (numberOfCards > 0 && numberOfCards < 6)
            {
                // Set the specifications of the cards.
                MAX_IO_CARD = numberOfCards;
                MAX_IO_CHIP = numberOfCards*2;
                MAX_IO_CHANNEL = numberOfCards*16;
                MAX_DAC_CHANNEL = numberOfCards*8;
                MAX_AD_CHANNEL = numberOfCards*4;
                MAX_DA_CHANNEL = numberOfCards;

                // Modify the dimensions of the arrays.
                IO = new int[MAX_IO_CHANNEL + 1];
                DAC = new int[MAX_DAC_CHANNEL + 1];
                AD = new int[MAX_AD_CHANNEL + 1];
                DA = new int[MAX_DA_CHANNEL + 1];
                IOconfig = new int[MAX_IO_CHIP];
                IOdata = new int[MAX_IO_CHIP];
                ADDAchipCode = new int[MAX_IO_CARD];
                DACchipCode = new int[MAX_IO_CARD];
                IOchipCode = new int[MAX_IO_CHIP];

                init();
            }
            else
            {
                throw new ArgumentException("Please specify a correct number of K8000 cards: 1-5.");
            }

            ERROR_ON_FAILURE = errorOnI2Cfailure;
        }
    #endregion



    
    #region "Class Information"
        /// <summary>
        /// lastI2cTransactionFailed will indicate if the last I2C action resulted in an error.
        /// This function can be used to detect if there is a problem with some chips on the card.
        /// </summary>
        /// <returns>A boolean which is set to true if the last I2C action failed.</returns>
        private bool lastI2cTransactionFailed()
        {
            return ERROR_DURING_LAST_I2C_ACTION;
        }
    #endregion



    #region "I/O channels"
        /// <summary>
        /// configAllIOasOutput will configure all I/O channels as outputs.
        /// </summary>
        public void configAllIOasOutput()
        {
            for (int chipNo = 0; chipNo < MAX_IO_CHIP; chipNo++)
            {
                IOconfig[chipNo] = 0;
            }
            clearAllIO();
        }


        /// <summary>
        /// configAllIOasInput will configure all I/O channels as inputs.
        /// </summary>
        public void configAllIOasInput()
        {
            for (int chipNo = 0; chipNo < MAX_IO_CHIP; chipNo++)
            {
                IOconfig[chipNo] = 0;
                clearIOchip(chipNo);
                IOconfig[chipNo] = 255;
                readIOchip(chipNo);
            }
        }


        /// <summary>
        /// clearIOchip will clear all outputs of the selected chip
        /// </summary>
        /// <param name="chipNo">The number of the chip to set</param>
        public void clearIOchip(int chipNo)
        {
            IOOutput(chipNo, 0);
        }



        /// <summary>
        /// clearIOchip will set all outputs of the selected chip
        /// </summary>
        /// <param name="chipNo">The number of the chip to clear</param>
        public void setIOchip(int chipNo)
        {
            IOOutput(chipNo, 255);
        }



        /// <summary>
        /// configIOchipAsOutput will config the selected chip as outputs
        /// </summary>
        /// <param name="chipNo">The number of the chip you wish to configure as output</param>
        public void configIOchipAsOutput(int chipNo)
        {
            IOconfig[chipNo] = 0;
            clearIOchip(chipNo);
        }



        /// <summary>
        /// configIOchipAsOutput will config the selected chip as inputs
        /// </summary>
        /// <param name="chipNo">The number of the chip you wish to configure as input</param>
        public void configIOchipAsInput(int chipNo)
        {
            IOconfig[chipNo] = 0;
            clearIOchip(chipNo);
            IOconfig[chipNo] = 255;
            readIOchip(chipNo);
        }



        /// <summary>
        /// configIOchannelAsInput will set an individual channel as I/O input.
        /// </summary>
        /// <param name="channelNO">The number of the channel to set as an input</param>
        public void configIOchannelAsInput(int channelNO)
        {   // calculate the channels
            int channel, chipNo;
            chipNo = (channelNO - 1) / 8;
            channel = (channelNO - 1) % 8;

            // write the info to the card
            IOconfig[chipNo] = IOconfig[chipNo] & ~(this.SHL(channel));
            clearIOchannel(channelNO);
            IOconfig[chipNo] = IOconfig[chipNo] | this.SHL(channel);
            readIOchannel(channelNO);
        }



        /// <summary>
        /// readIOchannel will read the value of an input channel
        /// </summary>
        /// <param name="channelNO">The number of the channel to read</param>
        public void readIOchannel(int channelNO)
        {
            int chipNo;
            chipNo = (channelNO - 1) / 8;
            readIOchip(chipNo);
        }



        /// <summary>
        /// configIOchannelAsInput will set an individual channel as I/O output
        /// </summary>
        /// <param name="channelNO">The number of the channel to set as an output</param>
        public void ConfigIOchannelAsOutput(int channelNO)
        {
            int channel, chipNo;
            chipNo = (channelNO - 1) / 8;
            channel = (channelNO - 1) % 8;
            IOconfig[chipNo] = IOconfig[chipNo] & ~(this.SHL(channel));
            clearIOchannel(channelNO);
        }



        /// <summary>
        /// updateAllIO will resend all the desired I/O outputs to the cards.
        /// </summary>
        public void updateAllIO()
        {
            for (int chipNo = 0; chipNo < MAX_IO_CHIP; chipNo++)
            {
                int temp;
                temp = IOdata[chipNo];
                IOOutput(chipNo, temp);
            }
        }



        /// <summary>
        /// updateIOchip will update the selected I/O chip so it reflects the current value of the IO[] array.
        /// </summary>
        /// <param name="chipNo">The number of the chip to set.</param>
        public void updateIOchip(int chipNo)
        {
            int temp;
            temp = IOdata[chipNo];
            IOOutput(chipNo, temp);
        }



        /// <summary>
        /// setAllIO will clear all the I/O channels.
        /// </summary>
        public void clearAllIO()
        {
            for (int chipNo = 0; chipNo < MAX_IO_CHIP; chipNo++)
            {
                IOOutput(chipNo, 0);
            }
        }



        /// <summary>
        /// setAllIO will set all the I/O channels.
        /// </summary>
        public void setAllIO()
        {
            for (int chipNo = 0; chipNo < MAX_IO_CHIP; chipNo++)
            {
                IOOutput(chipNo, 255);
            }
        }



        /// <summary>
        /// IOOutput will send the specified byte to the I/O chip.
        /// </summary>
        /// <param name="chipNo">The number of the chip you want to send the data to</param>
        /// <param name="Datavar">The data to send (8-bit word).</param>
        public void IOOutput(int chipNo, int Datavar)
        {
            int temp, temp2, iomask;

            // Step 1: calculate the neccesary bit patterns
            temp = Datavar;
            temp2 = ~temp;
            iomask = IOconfig[chipNo];
            Datavar = temp2 | iomask;

            // Step 2: Send the data to the K8000
            byte[] cmdToSend = new byte[1];
            cmdToSend[0] = (byte)Datavar;
            i2cWrite(IOchipCode[chipNo], cmdToSend);

            // Step 3: Update the IO chip data.
            IOdata[chipNo] = (IOdata[chipNo] & IOconfig[chipNo]) | ~Datavar;

            // Step 4: Update the I/O array
            int startchannel = chipNo * 8 + 1;
            for (int channel = 0; channel < 8; channel++)
            {
                int bitToCheck = (int)System.Math.Pow(2, channel);

                if ((IOdata[chipNo] & bitToCheck) == bitToCheck)
                    IO[startchannel + channel] = 1;
                else
                    IO[startchannel + channel] = 0;
            }
        }



        /// <summary>
        /// clearIOchannel will set the selected I/O channel.
        /// </summary>
        /// <param name="channel">the channel to set</param>
        public void setIOchannel(int channel)
        {
            int chipNo;
            chipNo = (channel - 1) / 8;
            int numOfchip = chipNo;
            int Datavar;
            int sendchannel;
            sendchannel = (channel - 1) % 8;
            Datavar = SHL(sendchannel) | IOdata[chipNo];
            IOOutput(numOfchip, Datavar);
        }



        /// <summary>
        /// clearIOchannel will clear the selected I/O channel.
        /// </summary>
        /// <param name="channel">the channel to clear</param>
        public void clearIOchannel(int channel)
        {
            int chipNo;
            chipNo = (channel - 1) / 8;
            int numOfchip = chipNo;
            int Datavar;
            int sendchannel;
            sendchannel = (channel - 1) % 8;
            Datavar = IOdata[chipNo] & ~SHL(sendchannel);
            IOOutput(numOfchip, Datavar);
        }



        /// <summary>
        /// readAllIO will read all I/O channels
        /// </summary>
        public void readAllIO()
        {
            for (int chipNo = 0; chipNo < MAX_IO_CHIP; chipNo++)
            {
                readIOchip(chipNo);
            }
        }



        /// <summary>
        /// readIOchip will read all channels of the given chip and update the IO[] array.
        /// </summary>
        /// <param name="chipNo">The number of the chip you want to read</param>
        public void readIOchip(int chipNo)
        {
            // Phase 1: Send the data to the K8000
            IOdata[chipNo] = i2cRead(IOchipCode[chipNo]);
            IOdata[chipNo] = ~IOdata[chipNo];

            // Phase 2: Update the I/O array
            int startchannel = chipNo * 8 + 1;
            for (int channel = 0; channel < 8; channel++)
            {
                int pow = (int)System.Math.Pow(2, channel);

                if ((IOdata[chipNo] & pow) == pow)
                    IO[startchannel + channel] = 1;
                else
                    IO[startchannel + channel] = 0;
            }
        }



        /// <summary>
        /// The function SHL will generate a value where only one bit it set: this bit can
        /// be entered by the parameter numberOfBit. If you specify the number 4, it will
        /// generate the binary: 00001000 (digital: 8).
        /// </summary>
        /// <param name="numberOfBit">the number of the bit you want to set. 0 equals the LSB,
        /// 8 equals the MSB.</param>
        /// <returns></returns>
        private int SHL(int numberOfBit)
        {
            int result = 1;

            for (int i = 0; i < numberOfBit; i++)
            {
                result <<= 1;
            }

            return result;
        }
    #endregion



    #region "AD channels"
        /// <summary>
        /// readAllAD will read the value of all AD chips.
        /// </summary>
        public void readAllAD()
        {
            for (int cardNo = 0; cardNo < MAX_IO_CARD; cardNo++)
            {
                readADchip(cardNo);
            }
        }



        /// <summary>
        /// readADchip will read a whole AD chip
        /// </summary>
        /// <param name="chipNo">The number of the chip you want to read</param>
        public void readADchip(int chipNo)
        {
            int chipAddress = ADDAchipCode[chipNo];

            byte[] responses;
            responses = i2cReadAD(chipAddress);

            int cardNo = (4 * chipNo);
            for (int i = 1; i < responses.Length; i++)
            {
                    AD[(i + cardNo)] = responses[i];
            }
        }



        /// <summary>
        /// readADchannel will read the value of the specified A/D channel
        /// </summary>
        /// <param name="channelNo">The number of the channel you want to read</param>
        public void readADchannel(int channelNo)
        {
            int chipNo = (channelNo - 1) / 4;
            readADchip(chipNo);
        }

    #endregion



    #region "DAC channels"
        /// <summary>
        /// updateDAchannel will resend the information to the selected DAC channel.
        /// </summary>
        /// <param name="channelNo">The number of the DA channel you want to update.</param>
        public void updateDACchannel(int channelNo)
        {
            outputDACchannel(channelNo, DAC[channelNo]);
        }



        /// <summary>
        /// outputDACchannel will send a value to the specified DAC channel.
        /// </summary>
        /// <param name="channelNo">The channel you want to use</param>
        /// <param name="dataVar">The value you want to send (0-63)</param>
        public void outputDACchannel(int channelNo, int dataVar)
        {
            if (dataVar > 63)
            {
                dataVar = 63;
            }

            DAC[channelNo] = dataVar;

            int chipAddress = DACchipCode[(channelNo - 1) / 8];
            byte[] values = new byte[2];
            int cmd = 240 | ((channelNo - 1) % 8);

            values[0] = (byte)cmd;          // control byte
            values[1] = (byte)dataVar;      // actual data.
            i2cWrite(chipAddress, values);
        }



        /// <summary>
        /// updateDACchip will resend all the data to the selected chip
        /// </summary>
        /// <param name="chipNo">The number of the chip you want to update.</param>
        public void updateDACchip(int chipNo)
        {
            int chipAddress = DACchipCode[chipNo];

            byte[] values = new byte[9];
            values[0] = 0;                  // control byte

            int channel;
            int temp;

            channel = chipNo * 8;
            for (int k = 1; k < 9; k++)
            {
                if (DAC[channel + k] > 63)
                {
                    DAC[channel + k] = 63;
                }
                temp = DAC[channel + k];
                values[k] = (byte)temp;
            }

            i2cWrite(chipAddress, values);
        }



        /// <summary>
        /// updateAllDAC will resend all DAC values to the card.
        /// </summary>
        public void updateAllDAC()
        {
            for (int cardNo = 0; cardNo < MAX_IO_CARD; cardNo++)
            {
                updateDACchip(cardNo);
            }
        }



        /// <summary>
        /// setAllDAC will set the DAC outputs of all channels to the maximum value (63)
        /// </summary>
        public void setAllDAC()
        {
            for (int channelNo = 1; channelNo < MAX_DAC_CHANNEL + 1; channelNo++)
            {
                DAC[channelNo] = 63;
            }
            updateAllDAC();
        }



        /// <summary>
        /// clearAllDAC will clear all the DAC outputs.
        /// </summary>
        public void clearAllDAC()
        {
            for (int channelNo = 1; channelNo < MAX_DAC_CHANNEL + 1; channelNo++)
            {
                DAC[channelNo] = 0;
            }
            updateAllDAC();
        }



        /// <summary>
        /// clearDACchannel will clear the output of the selected DAC channel (0)
        /// </summary>
        /// <param name="channelNo">The channel to clear</param>
        public void clearDACchannel(int channelNo)
        {
            outputDACchannel(channelNo, 0);
        }



        /// <summary>
        /// setDACchannel will set the output of the selected DAC channel to the maximum value (63)
        /// </summary>
        /// <param name="channelNo">The channel to set</param>
        public void setDACchannel(int channelNo)
        {
            outputDACchannel(channelNo, 63);
        }



        /// <summary>
        /// clearDACchip will clear all the outputs of the selected DAC chip.
        /// </summary>
        /// <param name="chipNo">The number of the chip to clear</param>
        public void clearDACchip(int chipNo)
        {
            int channel;

            channel = chipNo * 8;

            for (int i = 1; i < 9; i++)
            {
                DAC[channel + i] = 0;
            }

            updateDACchip(chipNo);
        }



        /// <summary>
        /// setDACchip will set all the outputs of the selected DAC chip to the maximum value (63).
        /// </summary>
        /// <param name="chipNo">The number of the chip to set</param>
        public void setDACchip(int chipNo)
        {
            int channel;

            channel = chipNo * 8;

            for (int i = 1; i < 9; i++)
            {
                DAC[channel + i] = 63;
            }

            updateDACchip(chipNo);
        }

    #endregion



    #region "DA channels"
        /// <summary>
        /// outputDAchannel will the specified value to the specified DA channel.
        /// </summary>
        /// <param name="channelNo">The number of the DA channel</param>
        /// <param name="dataVar">The value to send (0-255)</param>
        public void outputDAchannel(int channelNo, int dataVar)
        {
            int temp;
            DA[channelNo] = dataVar;

            temp = ADDAchipCode[channelNo - 1];

            byte[] values = new byte[2];
            values[0] = (byte)64;           // control byte
            values[1] = (byte)dataVar;      // actual data.

            i2cWrite(temp, values);
        }



        /// <summary>
        /// clearAllDA will set the outputs of all DA channels to 0.
        /// </summary>
        public void clearAllDA()
        {
            for (int channelNo = 1; channelNo < MAX_DA_CHANNEL + 1; channelNo++)
            {
                outputDAchannel(channelNo, 0);
            }
        }



        /// <summary>
        /// clearDAchannel will set the output of the selected DA channel to 255.
        /// </summary>
        /// <param name="channelNo">The DA channel to set</param>
        public void setDAchannel(int channelNo)
        {
            outputDAchannel(channelNo, 255);
        }



        /// <summary>
        /// clearDAchannel will set the output of the selected DA channel to 0.
        /// </summary>
        /// <param name="channelNo">The DA channel to clear</param>
        public void clearDAchannel(int channelNo)
        {
            outputDAchannel(channelNo, 0);
        }



        /// <summary>
        /// setAllDA will set all DA channels to the maximum value (255)
        /// </summary>
        public void setAllDA()
        {
            for (int channelNo = 1; channelNo < MAX_DA_CHANNEL + 1; channelNo++)
            {
                outputDAchannel(channelNo, 255);
            }
        }



        /// <summary>
        /// updateAllDA will update the value of all DA chnanels
        /// </summary>
        public void updateAllDA()
        {
            for (int channelNo = 1; channelNo < MAX_DA_CHANNEL + 1; channelNo++)
            {
                outputDAchannel(channelNo, DA[channelNo]);
            }
        }



        /// <summary>
        /// updateDAchannel will resend the information to the selected DA channel.
        /// </summary>
        /// <param name="channelNo">The number of the DA channel you want to update.</param>
        public void updateDAchannel(int channelNo)
        {
            outputDAchannel(channelNo, DA[channelNo]);
        }
    #endregion



    #region "General update & read functions"
        /// <summary>
        /// readCard will read all the chips of the selected card.
        /// </summary>
        /// <param name="cardNo">The number of the card to read</param>
        public void readCard(int cardNo)
        {
            int chipNo;
            chipNo = cardNo * 2;
            readIOchip(chipNo);
            readIOchip(chipNo + 1);
            readADchip(chipNo);
        }



        /// <summary>
        /// updateAll will reread all the channels on your card.
        /// </summary>
        public void updateAll()
        {
            updateAllIO();
            updateAllDAC();
            updateAllDA();
        }



        /// <summary>
        /// updateCard will update all chips on the card to match the data that is inside the arrays in this class.
        /// </summary>
        /// <param name="cardNo">The number of the card to update</param>
        public void updateCard(int cardNo)
        {
            int chipNo;
            chipNo = cardNo * 2;
            updateIOchip(chipNo);
            updateIOchip(chipNo + 1);
            updateDACchip(cardNo);
            updateDAchannel(cardNo + 1);
        }



        /// <summary>
        /// readAll will read all chips on your cards.
        /// </summary>
        public void readAll()
        {
            readAllIO();
            readAllAD();
        }
    #endregion



    #region "I2C Procedures"
        /// <summary>
        /// i2cWrite will write the specified data to the chip with the given address.
        /// </summary>
        /// <param name="address">The address of the I2C chip to write to.</param>
        /// <param name="value">A byte array containing all the bytes to send to the chip.</param>
        public void i2cWrite(int address, byte[] value)
        {
            //create I2C object
            I2CDevice.Configuration con = new I2CDevice.Configuration((ushort)address, I2C_BUS_FREQUENCY);
            I2CDevice MyI2C = new I2CDevice(con);

            // prepare the actions
            I2CDevice.I2CTransaction[] xActions = new I2CDevice.I2CTransaction[1];
            xActions[0] = I2CDevice.CreateWriteTransaction(value);

            // Generate an error when the communication fails.
            if (MyI2C.Execute(xActions, 1000) == 0)
            {   // an error occured
                ERROR_DURING_LAST_I2C_ACTION = true;

                if (ERROR_ON_FAILURE == true)
                    throw new ArgumentException("I2C device not found: timeout!");
            }
            else
            {
                ERROR_DURING_LAST_I2C_ACTION = false;
            }

            // Close the I2C bus
            MyI2C.Dispose();
            MyI2C = null;
        }



        /// <summary>
        /// i2cRead will read the I2C chip with the given address. It only works with I2C chips
        /// that responds with a single byte value.
        /// </summary>
        /// <param name="address">The address of the chip to read</param>
        /// <returns></returns>
        public int i2cRead(int address)
        {
            //create I2C object
            I2CDevice.Configuration con;
            I2CDevice MyI2C;

            con = new I2CDevice.Configuration((ushort)address, I2C_BUS_FREQUENCY);
            MyI2C = new I2CDevice(con);

            // prepare the actions
            I2CDevice.I2CTransaction[] xActions = new I2CDevice.I2CTransaction[1];
            byte[] RegisterValue = new byte[1];
            xActions[0] = I2CDevice.CreateReadTransaction(RegisterValue);


            // Generate an error when the communication fails.
            if (MyI2C.Execute(xActions, 1000) == 0)
            {   // an error occured
                ERROR_DURING_LAST_I2C_ACTION = true; 

                if (ERROR_ON_FAILURE == true)
                    throw new ArgumentException("I2C device not found: timeout!");
            }
            else
            {
                ERROR_DURING_LAST_I2C_ACTION = false; 
            }
               
            // Close the I2C bus
            MyI2C.Dispose();
            MyI2C = null;
            con = null;
            return (int)RegisterValue[0];            
        }


        /// <summary>
        /// i2cReadAD will read the analog channels of the selected chip.
        /// </summary>
        /// <param name="address">The address of the chip to read</param>
        /// <returns></returns>
        public byte[] i2cReadAD(int address)
        {
            //create I2C object
            I2CDevice.Configuration con = new I2CDevice.Configuration((ushort)address, I2C_BUS_FREQUENCY);
            I2CDevice MyI2C = new I2CDevice(con);

            // prepare the actions
            // TODO: this function might still have errors.
            I2CDevice.I2CTransaction[] xActions = new I2CDevice.I2CTransaction[2];
            byte[] RegisterValue = new byte[5] { 0, 0, 0, 0, 0 };

            xActions[0] = I2CDevice.CreateWriteTransaction(new byte[1] { 68 });
            xActions[1] = I2CDevice.CreateReadTransaction(RegisterValue);

            // Generate an error when the communication fails.
            if (MyI2C.Execute(xActions, 1000) == 0)
            {   // an error occured
                ERROR_DURING_LAST_I2C_ACTION = true;

                if (ERROR_ON_FAILURE == true)
                    throw new ArgumentException("I2C device not found: timeout!");
            }
            else
            {
                ERROR_DURING_LAST_I2C_ACTION = false;
            }

            // Close the I2C bus
            MyI2C.Dispose();
            MyI2C = null;

            return RegisterValue;
        }
    #endregion

}
