using System;
using Microsoft.SPOT;

/// <summary>
/// The command IOCONFIG is used to configure the I/O channels as
/// inputs or as outputs.
/// 
/// Usage: IOCONFIG [channelconfig]
///     - channelconfig is an string value containing a 16-bit pattern. Each of the bits specifies
///       the state (input/output) of each channel. The first bit represents I/O channel 1, the
///       last bit represents I/O channel 16.
///       
///       a channel is configured as I/O input when the relevant bit is set to 1, otherwise it is
///       set to 0.
///       
///  You should call this function with care: executing the IOCONFIG command will clear the
///  value of all I/O channels. I advise you to use this command in combination with the
///  INIT command. This last command allows you to set/read the initialisation state. You should
///  use the following procedure at the beginning of your communications:
///  
///  1) Call the init command to see the initialisation status                 "INIT 2"
///  2) The command responds with the current status                           "+OK 0"  or  "+OK 1"
///  3) The response +OK 0 indicates the initialisation is not done yet,
///     then do the initialisation (step 4) or skip to step 6.
///  4) Call the IOconfig command                                              "IOCONFIG 0000000000000000"
///  5) Set the initialisation status                                          "INIT 1"
///  6) You can now proceed with the normal communications
///  
/// </summary>
class CmdIOConfig : Command
{

    /// <summary>
    /// class constructor
    /// </summary>
    public CmdIOConfig()
    {
        keyword = "IOCONFIG";

        // add all the parameters for this object
        addParameter("channelconfig", parameterType.STRING, 16);
    }



    /// <summary>
    /// executeCommand() will execute the Cmd... class on the K8000 card
    /// </summary>
    /// <returns>A boolean to indicate if the execution succeeded</returns>
    public override string executeCommand()
    {
        String channelConfig = getStringParameter("channelconfig");
        if (channelConfig.Length == 16)
        {

            for (int i = 0; i < 16; i++)
            {
                string curConfig = channelConfig.Substring(i, 1);
                if (curConfig == "1")
                    K8000.ConfigIOchannelAsOutput (i+1);
                else
                    K8000.configIOchannelAsInput(i+1);
            }
            return "";
        }
        else
        {
            throw new Exception("invalid length");
        }
    }
}

