using System;
using Microsoft.SPOT;


/// <summary>
/// The command INIT was introduced to specify if the K8000 was already initialized
/// or not. Our K8000 needs to be initialized the first time it is used. During this
/// initialisation all channals are reset and the I/O configuration is set (output/input).
/// This only has to be done once, so the INIT keeps track of the current status.
/// 
/// Init knows 2 states:
/// 1   : INITIALIZED
/// 0   : NOT INITIALIZED
/// 
/// 
/// Usage: INIT [command]
///     - command is the desired state and can be 0 or 1. This value will be remembered
///     by the server.
///     
/// 
/// You can also retrieve the initialisation state by calling the command INIT 2.
/// Then it will respond with the current initialisation state:
/// 
/// +OK 1
/// or
/// +OK 0
/// 
/// 
/// </summary>
class CmdSetInitialized : Command
{

    const int INITIALIZED_SET = 1;
    const int INITIALIZED_CLEAR = 0;
    const int INITIALIZED_REQUEST = 2;
    private int currentInitialisationLevel = 0;


    /// <summary>
    /// class constructor
    /// </summary>
    public CmdSetInitialized()
    {
        keyword = "INIT";
        addParameter ("command", parameterType.INTEGER, 0,2);
    }



    /// <summary>
    /// executeCommand() will execute the Cmd... class on the K8000 card
    /// </summary>
    /// <returns>A boolean to indicate if the execution succeeded</returns>
    public override string executeCommand()
    {
        int commandType = getIntParameter("command");

        // get the initialisation status directy from the server
        if (K8000_Server.initialized)
            currentInitialisationLevel = 1;
        else
            currentInitialisationLevel = 0;

        // evaluate initialisation
        switch (commandType)
        {
            case INITIALIZED_SET:
                currentInitialisationLevel = INITIALIZED_SET;
                K8000_Server.initialized = true;
                break;
            case INITIALIZED_CLEAR:
                K8000_Server.initialized = false; 
                currentInitialisationLevel = INITIALIZED_CLEAR;
                break;
        }


        return "" + currentInitialisationLevel;
    }
}

