using System;
using Microsoft.SPOT;


/// <summary>
/// The class Parameter respresents a single parameter.
/// 
/// Each Cmd... class will have to add x instances of the Parameter class: each for one
/// parameter that the command needs. This parameter object defines the expected type,
/// max length,min value, max value, ...
/// </summary>
class Parameter
{
    // overview of the available commands
    private string _name;
    private Command.parameterType _parameterType;
    private int _maxLength=0;
    private int _minLength=0; 
    private int _min;
    private int _max;
    private int _intValue=-1;
    private string _stringValue="";
    private string _parseError;



    #region "properties"
    /// <summary>
    /// name contains the name of the parameter
    /// </summary>
    public string name
    {
        get { return _name; }
        set
        {
            _name = value;
        }
    }

    /// <summary>
    /// parameterType contains the type of the parameter (see parameterType in the 
    /// Command class to see the allowed parameter types).
    /// </summary>
    public Command.parameterType parameterType
    {
        get { return _parameterType; }
        set
        {
            _parameterType = value;
        }
    }


    /// <summary>
    /// maxLength contains the maximal length of a String parameter
    /// </summary>
    public int maxLength
    {
        get { return _maxLength; }
        set
        {
            _maxLength = value;
        }
    }


    /// <summary>
    /// minLength contains the maximal length of a String parameter
    /// </summary>
    public int minLength
    {
        get { return _minLength; }
        set
        {
            _minLength = value;
        }
    }


    /// <summary>
    /// min contains the minimal value of an Integer parameter
    /// </summary>
    public int min
    {
        get { return _min; }
        set
        {
            _min = value;
        }
    }



    /// <summary>
    /// max contains the maximal value of an Integer parameter
    /// </summary>
    public int max
    {
        get { return _max; }
        set
        {
            _max = value;
        }
    }



    /// <summary>
    /// intValue contains the integer value of the parameter
    /// </summary>
    public int intValue
    {
        get { return _intValue; }
        set
        {
            _intValue = value;
        }
    }



    /// <summary>
    /// stringValue contains the string value of the parameter
    /// </summary>
    public string stringValue
    {
        get { return _stringValue; }
        set
        {
            _stringValue = value;
        }
    }


    /// <summary>
    /// parseError is a boolean to indicate if the parsing of the
    /// parameter succeeded or not.
    /// </summary>
    public string parseError
    {
        get { return _parseError; }
        set
        {
            _parseError = value;
        }
    }
    #endregion





    /// <summary>
    /// assignIfPossible() allows to assign a value to the parameter object. This is done by passing a String 
    /// to this function (keep in mind the server works with texts strings: all this data is treated as texts
    /// until this function converts it to integers).
    /// 
    /// After assigning, the value of the parameter will be stored into the numericValue or stringValue 
    /// properties of the class (depening on the type of parameter).
    /// </summary>
    /// <param name="value">the parameter value to process</param>
    /// <returns>A boolean to indicate if the conversion went well</returns>
    public bool assignIfPossible(string value) {

        bool response = false;

        switch (parameterType)
        {
            // processing of an Integer value
            case Command.parameterType.INTEGER:
                try
                {
                    int numericValue = int.Parse(value);

                    if (numericValue >= min && numericValue <= max)
                    {   // integer AND right range
                        intValue = numericValue; 
                        response = true; 
                    }
                    else
                    {   // integer BUT wrong range
                        response = false;
                    }
                }
                catch (Exception e)
                {   // no integer. We don't care about the error code.
                    response = false; 
                }
            
                break;


            // processing of a String value
            case Command.parameterType.STRING:
                if (value.Length <= maxLength)
                {   // TODO: implement min length
                    stringValue = value;
                    response = true;
                }
                else
                {
                    response = false; 
                }

                break;
        }

        return response;
    }




    /// <summary>
    /// Parameter() is the class constructor to create an instance of the class that represents
    /// a String value
    /// </summary>
    /// <param name="name">name of the parameter</param>
    /// <param name="typeOfParameter">Type (should be string!)</param>
    /// <param name="maxLength">the maximal length of the String</param>
    public Parameter(string name, Command.parameterType typeOfParameter, int maxLength) {
        this.name = name;
        this.parameterType = typeOfParameter;
        this.maxLength = maxLength;
    }



    /// <summary>
    /// Parameter() is the class constructor to create an instance of the class that represents
    /// an Integer value
    /// </summary>
    /// <param name="name">name of the parameter</param>
    /// <param name="typeOfParameter">Type (should be string!)</param>
    /// <param name="min">the minimal allowed value of the integer value</param>
    /// <param name="max">the maximal allowed value of the integer value</param>
    public Parameter(string name, Command.parameterType typeOfParameter, int min, int max) {
        this.name = name;
        this._parameterType = typeOfParameter;
        this.min = min;
        this.max = max;
    }
}
