using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Net;
using GHIElectronics.NETMF.Net.NetworkInformation;
using GHIElectronics.NETMF.Net.Sockets;


/// <summary>
/// The K8000_Server represents the server class. This class is listening to the TCP/IP 
/// server port. Each connection will accepted in a separate background thread.
/// </summary>
public class K8000_Server
{
    // Initial server properties. You don't need to change them here: just change them
    // in the Program.cs source file.
    private int _serverPort = 23;
    private byte[] _ip = { 192, 168, 2, 150 };
    private byte[] _subnet = { 255, 255, 255, 0 };
    private byte[] _gateway = { 192, 168, 1, 1 };
    private byte[] _mac = { 0x00, 0x88, 0x98, 0x90, 0xD4, 0xE0 };
    private static string _password = "bart!";
    private static Velleman_K8000 _K8000;
    private static bool _initialized = false; 


    // This section of the code contains the network properties. They can be used to pass the information from the Program.cs file
    #region "Properties"
    /// <summary>
    /// This property gets/sets the initialized status
    /// </summary>
    public static bool initialized
    {
        get { return _initialized; }
        set
        {
            _initialized = value;
        }
    }

    /// <summary>
    /// This property gets/sets the K8000 object. Only 1 may exist!
    /// </summary>
    public Velleman_K8000 K8000
    {
        get { return _K8000; }
        set
        {
            _K8000 = value;
        }
    }


    /// <summary>
    /// This function will get the K8000 object.
    /// </summary>
    /// <returns></returns>
    public static Velleman_K8000 getK8000()
    {
        return _K8000;
    }


    /// <summary>
    /// This property gets/sets the password for access to the server.
    /// </summary>
    public string password
    {
        get { return _password; }
        set
        {
            _password = value;
        }
    }


    /// <summary>
    /// This property gets the IP address that was set in the K8000_Server class.
    /// It is a static property, so it can be called from any other class in the
    /// project without having access to a K8000_Server object (instance of the
    /// class).
    /// </summary>
    /// <returns></returns>
    public static string getStaticPassword()
    {
        return _password;
    }

    /// <summary>
    /// Sets the TCP/IP port the server is listening to. You should select a number 
    /// between 1 and 65535. 
    /// </summary>
    public int serverPort {
        get { return _serverPort; }
        set
        {
            _serverPort = value;
        }
    }

    /// <summary>
    /// Sets the IP address of your FEZ device. This address should be unique on
    /// your network.
    /// </summary>
    public byte[] ip
    {
        get { return _ip; }
        set
        {
            _ip = value;
        }
    }

    /// <summary>
    /// Sets the Subnet mask of your network. The subnet mask looks like 
    /// 255.255.255.0 and should match the subnet of the other network
    /// devices in your network. Best practice is to view configured subnet
    /// mask of your router.
    /// </summary>
    public byte[] subnet
    {
        get { return _subnet; }
        set
        {
            _subnet = value;
        }
    }

    /// <summary>
    /// Sets the Gateway of your network. This is the IP address of the 
    /// router that has access to your WAN (Internet).
    /// </summary>
    public byte[] gateway
    {
        get { return _gateway; }
        set
        {
            _gateway = value;
        }
    }

    /// <summary>
    /// Set the MAC address for the FEZ Connect Shield's network module.
    /// A MAC should be unique on your network.
    /// </summary>
    public byte[] mac
    {
        get { return _mac; }
        set
        {
            _mac = value;
        }
    }
    #endregion



    /// <summary>
    /// runServer() will execute the server program.
    /// </summary>
    public void runServer ()
    {
        // here we activate the W5100 module of the FEZ Connect Shield
        WIZnet_W5100.Enable(SPI.SPI_module.SPI1, (Cpu.Pin)FEZ_Pin.Digital.Di10,
        (Cpu.Pin)FEZ_Pin.Digital.Di7, true);
        NetworkInterface.EnableStaticIP(ip, subnet, gateway, mac);
        NetworkInterface.EnableStaticDns(gateway);

        // First we define a server socket that is listening for any incoming TCP/IP connections. 
        // The FEZ hardware supports up to 4 sockets so we can only have 3 incoming connections 
        // at a time! Actions should be taken to avoid all availabe sockets are locked forever!
        Socket server = new Socket(AddressFamily.InterNetwork, 
                                   SocketType.Stream,
                                   ProtocolType.Tcp);

        IPEndPoint localEndPoint = new IPEndPoint(IPAddress.Any, serverPort);
        server.Bind(localEndPoint);
        server.Listen(1);


        // We now start an indefinately loop. The loop will scan for new connection requests
        // and accept the new connection. The server stops accepting new connections when there
        // are already 2 active connections. This is neccesary because the FEZ Connect Shield
        // supports just a few connections at the same time.
        while (true)
        {      
            // we absolutely need to put a try/catch code in the loop. Otherwise our server could
            // crash completely when a certain error occurs.
            try
            {
                // Wait for a client to connect and assign it to the newConnectionSocket.
                Socket newConnectionSocket = server.Accept();

                // TODO: this code does not work properly. New connections should be rejected and
                // no errors should occur!
                if (ServerConnection.getActiveConnectionsCounter() < 2)
                {
                    new ServerConnection(newConnectionSocket);
                }
                else
                {
                    newConnectionSocket.Close();
                    Debug.Print("connection rejected!");
                }
            }
            catch (Exception e)
            {
                // an error occured. Probably the server sockets are all used. We just do nothing.
                Debug.Print("An exception occured: " + e.Message);
            }
        }
    }
}
