using System;
using System.Threading;
using System.Text;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Net;
using GHIElectronics.NETMF.Net.NetworkInformation;
using GHIElectronics.NETMF.Net.Sockets;



/// <summary>
/// The ServerConnection class represents a single server connection with the FEZ 
/// server. A part of the class will be executed in a separate background thread
/// to allow multiple connections.
/// </summary>
class ServerConnection
{
    // this value holds the number of active connections
    private static int activeConnectionCount = 0;
    private static Command[] commandArray;
    byte[] prompt = Encoding.UTF8.GetBytes("+OK");
    private Socket clientSocket;
    private bool userLoggedIn = false;
    private bool requestDisconnect = false;
    private int failedAttempts = 0;
    public const int MAX_FAILED_ATTEMPTS = 10;




    /// <summary>
    /// The constructor for the ServerConnection which is executed when the class is
    /// initialized. It will put the variable Sock into the class variable and execute
    /// the Process in the background.
    /// </summary>
    /// <param name="Sock"></param>
    public ServerConnection(Socket Sock)
    {
        // increase the counter with the active connections (static field)
        activeConnectionCount++;

        // generate an array with all allowed server commands
        commandArray = CommandList.getCommands();

        //  run the connection in a background process (thread)
        Debug.Print("Client (#" + activeConnectionCount + ") connected...");
        clientSocket = Sock;
        new Thread(Process).Start();
    }




    /// <summary>
    /// getActiveConnectionsCounter() returns the number of server connections at a time.
    /// </summary>
    /// <returns>The number of server connections. In case of the FEZ Connect Shield this 
    /// could be 0,1 or 2.</returns>
    public static int getActiveConnectionsCounter()
    {
        return activeConnectionCount;
    }




    /// <summary>
    /// The function respond() will send a respond back to the client. An enter character will
    /// be appended to the end of the String.
    /// </summary>
    /// <param name="txt">The string to send</param>
    private void respond(string txt)
    {
        txt += "\r\n";

        try
        {
            clientSocket.Send(Encoding.UTF8.GetBytes(txt), 0, txt.Length, SocketFlags.None);
        }
        catch (Exception e)
        {
            Debug.Print("error while sending response to the client: " + e.Message); 
        }
    }




    /// <summary>
    /// sendCommandToK8000() will send a command to the K8000 card. It accepts an array
    /// of a String. Each element contains the separate words of the received command.
    /// </summary>
    /// <param name="segments">An array with elements</param>
    public void sendCommandToK8000(String[] segments)
    {
        // initialisation of the code
        int commandFound = -1;
        bool commandResult = true;
        String commandError = "";
        String commandResponse = "";
        Command.serverFlags serverFlag = Command.serverFlags.NONE;

        // TODO: clear this up!
        // I implemented this code because you may only enter commands after a positive login.
        int maxCommand = commandArray.Length;
        if (!userLoggedIn) maxCommand = 1;

        // here we try to determine the current command
        for (int i = 0; i < maxCommand; i++)
        {
            Command currentCommand = commandArray[i];           

            if (currentCommand.keyword == segments[0])
            {   // command found
                commandFound = i;
                //currentCommand.K8000 = k8000;   // pass the K8000 object to the card
                currentCommand.K8000 = K8000_Server.getK8000();
                commandResult = currentCommand.executeCommand(segments);
                commandError = currentCommand.error;
                commandResponse = currentCommand.response;
                serverFlag = currentCommand.serverflag;
            }
        }


        // the command was not found, show an error message!
        if (commandFound == -1)
        {
            respond("+ERR Unknown command");
        }
        // the command was found
        else
        {
            if (commandResult)
            {
                if (commandError != "")
                { // an error occured while executing the command
                    respond("+ERR " + commandError);
                }
                else
                {   // regular processing
                    if (commandResponse == "")
                        respond("+OK");
                    else
                        respond("+OK " + commandResponse);

                    if (serverFlag == Command.serverFlags.LOGGED_IN)
                        userLoggedIn = true;

                    if (serverFlag == Command.serverFlags.DISCONNECT_REQUEST)
                        requestDisconnect = true; 
                }






            }
            else
                if (commandError != "")
                { // an error occured while executing the command
                    respond("+ERR " + commandError);
                }
                else
                {
                    respond("+ERR " + commandResponse);
                }
        }
    }



    /// <summary>
    /// executeCommand() processes the received commands (from the Process background thread).
    /// 
    /// </summary>
    /// <param name="cmd">the received string line</param>
    private void executeCommand(string cmd)
    {
        // Split into separate words and put the first command in uppercase
        String[] segments = cmd.Split(' ');
        segments[0]=segments[0].ToUpper();

        // First we will verify if the user is logged in.
        if (!userLoggedIn && segments[0] != "PASS")
        {   // User not logged in, increase failedAttempts
            failedAttempts++;
            respond("+ERR Please login first!");
        }
        else
        {   // send the command to the K8000
            sendCommandToK8000(segments);
        }
    }







    /// <summary>
    /// The Process method represents a single session with a K8000 client. This code will
    /// be executed in a separate thread. It loops through an endless loop that is only
    /// exited when the connection is closed.
    /// 
    /// The connection can accept text messages containing all ASCII characters but it doesn't
    /// support the backspace command. You could open a Telnet session with the server, but
    /// the backspace won't work. Any command with a backspace is unrecognized.
    /// </summary>
    private void Process()
    {

        try
        {
            // method variables
            byte[] command = new byte[100]; // the received buffer can contain up to 100 characters.
            int index = 0;                  // index points to the current character in the command buffer.
            bool connected = true;          // set the initial connection status to CONNECTED.


            using (clientSocket)
            {
                // set the timeout for the receiver to 5 seconds (avoid server lockup during login)
                clientSocket.ReceiveTimeout = 5 * 1000 * 1000;

                // send the welcome message to the client.
                byte[] firstPrompt = Encoding.UTF8.GetBytes("+OK Fez-to-K8000 TCP server\r\n");
                clientSocket.Send(firstPrompt);

                // now we create a loop that is executed until the connection is closed.                
                while (connected && requestDisconnect==false)
                {
                    // step 1: verify the max failed attempts.
                    if (failedAttempts > MAX_FAILED_ATTEMPTS)
                    {   // max attempts reached!
                        clientSocket.Close();
                        connected = false;
                    }


                    // step 2: set the timeout
                    if (userLoggedIn)
                    {
                        clientSocket.ReceiveTimeout = 60 * 1000 * 1000;
                    }


                    // only proceed if the connection is still active
                    if (connected)
                    {
                        // step 2: read the incoming data and put it into the command buffer at position 'index'.
                        // the communication will be disconnected if there are 0 bytes received.
                        if (clientSocket.Receive(command, index, 1, SocketFlags.None) == 0)
                        {
                            clientSocket.Close();
                            connected = false;
                        }

                        // step3: verify if last received command contains a carriage return (enter). Then
                        // the command buffer is converted to a String and then executed. Otherwise the
                        // index pointer is increased with 1. The received data will be flushed if the
                        // number of received characters exceeds 100.
                        if (command[index] == '\r')
                        {
                            string cmd = new string(Encoding.UTF8.GetChars(command), 0, index);
                            executeCommand(cmd);
                            index = 0;
                        }
                        else if (command[index] == '\b')
                        {
                            index--;
                            if (index < 0) index = 0;
                        }
                        else
                        {
                            // check if the received character is an ASCII character. Only characters with a
                            // code between 32 and 126 are allowed.
                            if (command[index] >= 32 && command[index] <= 126)
                            {
                                index++;
                                if (index >= command.Length)
                                    index = 0;
                            }
                        }
                    }
                }
            }
        }
        catch (Exception e)
        {
            Debug.Print("Client disconnected by error=" + e.Message);
        }

        activeConnectionCount--;
        Debug.Print("Client disconnected. Connection Count=" + activeConnectionCount);
    }
}
